@@ -7,6 +7,7 @@
 #include "global.h"
 
 #include <QLayout>
+#include <QBoxLayout>
 #include <QLabel>
 
 #include <KLocale>
@@ -31,65 +32,99 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
     : QWidget( parent ),
     config( _config )
 {
+    int gridRow = 0;
     grid = new QGridLayout( this );
-//     grid->setContentsMargins( 6, 6, 6, 6 );
-//     grid->setSpacing( 6 );
 
     QHBoxLayout *topBox = new QHBoxLayout();
     grid->addLayout( topBox, 0, 0 );
 
-    QLabel *lFormat = new QLabel( i18n("Output format")+":", this );
+    QLabel *lFormat = new QLabel( i18n("Format:"), this );
     topBox->addWidget( lFormat );
     cFormat = new KComboBox( this );
     topBox->addWidget( cFormat );
-    cFormat->addItems( config->pluginLoader()->formatList(PluginLoader::Encode,PluginLoader::CompressionType(PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid)) );
+    cFormat->addItems( config->pluginLoader()->formatList(PluginLoader::Encode,PluginLoader::CompressionType(PluginLoader::InferiorQuality|PluginLoader::Lossy|PluginLoader::Lossless|PluginLoader::Hybrid)) );
     connect( cFormat, SIGNAL(activated(const QString&)), this, SLOT(formatChanged(const QString&)) );
 //     connect( cFormat, SIGNAL(activated(const QString&)), this, SLOT(somethingChanged()) );
 
     topBox->addStretch();
 
-    QLabel *lPlugin = new QLabel( i18n("Use Plugin")+":", this );
+    lPlugin = new QLabel( i18n("Use Plugin:"), this );
     topBox->addWidget( lPlugin );
     cPlugin = new KComboBox( this );
     topBox->addWidget( cPlugin );
     cPlugin->setSizeAdjustPolicy( QComboBox::AdjustToContents );
     connect( cPlugin, SIGNAL(activated(const QString&)), this, SLOT(encoderChanged(const QString&)) );
     connect( cPlugin, SIGNAL(activated(const QString&)), this, SLOT(somethingChanged()) );
+    pConfigurePlugin = new KPushButton( KIcon("configure"), "", this );
+    pConfigurePlugin->setFixedSize( cPlugin->sizeHint().height(), cPlugin->sizeHint().height() );
+    pConfigurePlugin->setFlat( true );
+    topBox->addWidget( pConfigurePlugin );
+    topBox->setStretchFactor( pConfigurePlugin, 1 );
+    connect( pConfigurePlugin, SIGNAL(clicked()), this, SLOT(configurePlugin()) );
 
     // draw a horizontal line
     QFrame *lineFrame = new QFrame( this );
     lineFrame->setFrameShape( QFrame::HLine );
     lineFrame->setFrameShadow( QFrame::Sunken );
+    lineFrame->setFixedHeight( 10 );
     grid->addWidget( lineFrame, 1, 0 );
 
     // prepare the plugin widget
     wPlugin = 0;
     grid->setRowStretch( 2, 1 );
+    grid->setRowMinimumHeight( 2, 20 );
+    gridRow = 3;
 
     // draw a horizontal line
     lineFrame = new QFrame( this );
     lineFrame->setFrameShape( QFrame::HLine );
     lineFrame->setFrameShadow( QFrame::Sunken );
-    grid->addWidget( lineFrame, 3, 0 );
+    lineFrame->setFixedHeight( 10 );
+    grid->addWidget( lineFrame, gridRow++, 0 );
+
+    int filterCount = 0;
+    foreach( QString pluginName, config->data.backends.enabledFilters )
+    {
+        FilterPlugin *plugin = qobject_cast<FilterPlugin*>(config->pluginLoader()->backendPluginByName(pluginName));
+        if( !plugin )
+            continue;
+
+        FilterWidget *widget = plugin->newFilterWidget();
+        if( !widget )
+            continue;
+
+        wFilter.insert( widget, plugin );
+        connect( widget, SIGNAL(optionsChanged()), this, SLOT(somethingChanged()) );
+        grid->addWidget( widget, gridRow++, 0 );
+        widget->show();
+        filterCount++;
+        break; // show only one widget at the moment
+    }
+    if( filterCount > 0 )
+    {
+        // draw a horizontal line
+        lineFrame = new QFrame( this );
+        lineFrame->setFrameShape( QFrame::HLine );
+        lineFrame->setFrameShadow( QFrame::Sunken );
+        lineFrame->setFixedHeight( 10 );
+        grid->addWidget( lineFrame, gridRow++, 0 );
+    }
 
     // the output directory
     QHBoxLayout *middleBox = new QHBoxLayout( );
-    grid->addLayout( middleBox, 4, 0 );
+    grid->addLayout( middleBox, gridRow++, 0 );
 
+    QLabel *lOutput = new QLabel( i18n("Destination:"), this );
+    middleBox->addWidget( lOutput );
     outputDirectory = new OutputDirectory( config, this );
     middleBox->addWidget( outputDirectory );
 
     QHBoxLayout *bottomBox = new QHBoxLayout();
-    grid->addLayout( bottomBox, 5, 0 );
+    grid->addLayout( bottomBox, gridRow++, 0 );
 
     cReplayGain = new QCheckBox( i18n("Calculate Replay Gain tags"), this );
     bottomBox->addWidget( cReplayGain );
     //connect( cReplayGain, SIGNAL(toggled(bool)), this, SLOT(somethingChanged()) );
-    bottomBox->addSpacing( 12 );
-//     cBpm = new QCheckBox( i18n("Calculate BPM tags"), this );
-//     cBpm->hide();
-//     bottomBox->addWidget( cBpm );
-    //connect( cBpm, SIGNAL(toggled(bool)), this, SLOT(somethingChanged()) );
     bottomBox->addStretch();
     lEstimSize = new QLabel( QString(QChar(8776))+"? B / min." );
     lEstimSize->hide(); // hide for now because most plugins report inaccurate data
@@ -99,39 +134,12 @@ OptionsDetailed::OptionsDetailed( Config* _config, QWidget* parent )
     pProfileSave->setFixedWidth( pProfileSave->height() );
     pProfileSave->setToolTip( i18n("Save current options as a profile") );
     connect( pProfileSave, SIGNAL(clicked()), this, SLOT(saveCustomProfile()) );
-//     pProfileLoad = new KPushButton( KIcon("document-open"), "", this );
     pProfileLoad = new QToolButton( this );
     bottomBox->addWidget( pProfileLoad );
     pProfileLoad->setIcon( KIcon("document-open") );
     pProfileLoad->setPopupMode( QToolButton::InstantPopup );
     pProfileLoad->setFixedWidth( pProfileLoad->height() );
     pProfileLoad->setToolTip( i18n("Load saved profiles") );
-//     connect( pProfileLoad, SIGNAL(clicked()), this, SLOT(saveProfile()) );
-
-/*
-    QList<CodecPlugin*> plugins = config->pluginLoader()->getAllCodecPlugins();
-    QList<QWidget*> widgets;
-    QWidget *newWidget;
-    for( int i=0; i<plugins.count(); i++ )
-    {
-        newWidget = plugins.at(i)->newCodecWidget();
-        if( newWidget )
-        {
-            grid->addWidget( newWidget, 2, 0 );
-            newWidget->show();
-        }
-        widgets += newWidget;
-    }
-
-    adjustSize();
-    size_hint = sizeHint();
-
-    for( int i=0; i<plugins.count(); i++ )
-    {
-        grid->removeWidget( widgets.at(i) );
-        plugins.at(i)->deleteCodecWidget( widgets.at(i) );
-    }
-*/
 }
 
 OptionsDetailed::~OptionsDetailed()
@@ -150,6 +158,18 @@ void OptionsDetailed::init()
 //     return size_hint;
 // }
 
+void OptionsDetailed::resetFilterOptions()
+{
+    for( int i=0; i<wFilter.size(); i++ )
+    {
+        FilterWidget *widget = wFilter.keys().at(i);
+        if( widget )
+        {
+            widget->setCurrentFilterOptions( 0 );
+        }
+    }
+}
+
 void OptionsDetailed::setReplayGainChecked( bool enabled )
 {
     cReplayGain->setChecked(enabled);
@@ -204,7 +224,6 @@ void OptionsDetailed::formatChanged( const QString& format )
         }
     }
     cPlugin->setCurrentIndex( 0 );
-//     cPlugin->setEnabled( format != "wav" );
 
     if( cPlugin->currentText() != oldEncoder )
     {
@@ -212,7 +231,36 @@ void OptionsDetailed::formatChanged( const QString& format )
     }
     else if( wPlugin )
     {
-        qobject_cast<CodecWidget*>(wPlugin)->setCurrentFormat( cFormat->currentText() );
+        wPlugin->setCurrentFormat( cFormat->currentText() );
+    }
+
+    lPlugin->setShown( format != "wav" );
+    cPlugin->setShown( format != "wav" );
+    pConfigurePlugin->setShown( format != "wav" );
+    if( wPlugin )
+        wPlugin->setShown( format != "wav" );
+
+    QStringList errorList;
+    cReplayGain->setEnabled( config->pluginLoader()->canReplayGain(cFormat->currentText(),currentPlugin,&errorList) );
+    if( !cReplayGain->isEnabled() )
+    {
+        QPalette notificationPalette = cReplayGain->palette();
+        notificationPalette.setColor( QPalette::Disabled, QPalette::WindowText, QColor(174,127,130) );
+        cReplayGain->setPalette( notificationPalette );
+
+        if( !errorList.isEmpty() )
+        {
+            errorList.prepend( i18n("Replay Gain is not supported for the %1 file format.\nPossible solutions are listed below.",cFormat->currentText()) );
+        }
+        else
+        {
+            errorList += i18n("Replay Gain is not supported for the %1 file format.\nPlease check your distribution's package manager in order to install an additional Replay Gain plugin.",cFormat->currentText());
+        }
+        cReplayGain->setToolTip( errorList.join("\n\n") );
+    }
+    else
+    {
+        cReplayGain->setToolTip( i18n("Replay Gain tags can tell your music player how loud a track is\nso it can adjust the volume to play all tracks with equal loudness.") );
     }
 
     somethingChanged();
@@ -231,42 +279,23 @@ void OptionsDetailed::encoderChanged( const QString& encoder )
     if( wPlugin )
     {
         grid->removeWidget( wPlugin );
-        disconnect( wPlugin, SIGNAL(somethingChanged()), 0, 0 );
+        disconnect( wPlugin, SIGNAL(optionsChanged()), 0, 0 );
         wPlugin = plugin->deleteCodecWidget( wPlugin );
     }
     wPlugin = plugin->newCodecWidget();
     if( wPlugin )
     {
-        connect( wPlugin, SIGNAL(somethingChanged()), this, SLOT(somethingChanged()) );
+        connect( wPlugin, SIGNAL(optionsChanged()), this, SLOT(somethingChanged()) );
         qobject_cast<CodecWidget*>(wPlugin)->setCurrentFormat( cFormat->currentText() );
         grid->addWidget( wPlugin, 2, 0 );
-        wPlugin->show();
-//         wPlugin->setVisible( cFormat->currentText() != "wav" );
     }
 
-    QStringList errorList;
-    cReplayGain->setEnabled( config->pluginLoader()->canReplayGain(cFormat->currentText(),currentPlugin,&errorList) );
-    if( !cReplayGain->isEnabled() )
-    {
-        QPalette notificationPalette = cReplayGain->palette();
-//         notificationPalette.setColor( QPalette::Disabled, QPalette::WindowText, QColor(181,96,101) );
-        notificationPalette.setColor( QPalette::Disabled, QPalette::WindowText, QColor(174,127,130) );
-        cReplayGain->setPalette( notificationPalette );
+    pConfigurePlugin->setEnabled( plugin->isConfigSupported(BackendPlugin::Encoder,"") );
 
-        if( !errorList.isEmpty() )
-        {
-            errorList.prepend( i18n("Replay Gain is not supported for the %1 file format.\nPossible solutions are listed below.",cFormat->currentText()) );
-        }
-        else
-        {
-            errorList += i18n("Replay Gain is not supported for the %1 file format.\nPlease check your distribution's package manager in order to install an additional Replay Gain plugin.",cFormat->currentText());
-        }
-        cReplayGain->setToolTip( errorList.join("\n\n") );
-    }
+    if( pConfigurePlugin->isEnabled() )
+        pConfigurePlugin->setToolTip( i18n("Configure %1 ...",encoder) );
     else
-    {
-        cReplayGain->setToolTip( "" );
-    }
+        pConfigurePlugin->setToolTip( "" );
 }
 
 void OptionsDetailed::somethingChanged()
@@ -274,7 +303,7 @@ void OptionsDetailed::somethingChanged()
     int dataRate = 0;
 
     if( wPlugin )
-        dataRate = qobject_cast<CodecWidget*>(wPlugin)->currentDataRate();
+        dataRate = wPlugin->currentDataRate();
 
     if( dataRate > 0 )
     {
@@ -291,13 +320,23 @@ void OptionsDetailed::somethingChanged()
     emit currentDataRateChanged( dataRate );
 }
 
-ConversionOptions *OptionsDetailed::currentConversionOptions()
+void OptionsDetailed::configurePlugin()
+{
+    CodecPlugin *plugin = (CodecPlugin*)config->pluginLoader()->backendPluginByName( cPlugin->currentText() );
+
+    if( plugin )
+    {
+        plugin->showConfigDialog( BackendPlugin::Encoder, "", this );
+    }
+}
+
+ConversionOptions *OptionsDetailed::currentConversionOptions( bool saveLastUsed )
 {
     ConversionOptions *options = 0;
 
     if( wPlugin && currentPlugin )
     {
-        options = qobject_cast<CodecWidget*>(wPlugin)->currentConversionOptions();
+        options = wPlugin->currentConversionOptions();
         if( options )
         {
             options->codecName = cFormat->currentText();
@@ -305,16 +344,33 @@ ConversionOptions *OptionsDetailed::currentConversionOptions()
                 options->pluginName = currentPlugin->name();
             else
                 options->pluginName = "";
-            options->profile = qobject_cast<CodecWidget*>(wPlugin)->currentProfile();
+            options->profile = wPlugin->currentProfile();
             options->outputDirectoryMode = outputDirectory->mode();
             options->outputDirectory = outputDirectory->directory();
             options->outputFilesystem = outputDirectory->filesystem();
-            options->replaygain = cReplayGain->isChecked();
-//             options->bpm = cBpm->isChecked();
+            options->replaygain = cReplayGain->isEnabled() && cReplayGain->isChecked();
 
-            config->data.general.lastProfile = currentProfile();
-            saveCustomProfile( true );
-            config->data.general.lastFormat = cFormat->currentText();
+            for( int i=0; i<wFilter.size(); i++ )
+            {
+                FilterWidget *widget = wFilter.keys().at(i);
+                FilterPlugin *plugin = wFilter.values().at(i);
+                if( widget && plugin )
+                {
+                    FilterOptions *filterOptions = widget->currentFilterOptions();
+                    if( filterOptions )
+                    {
+                        filterOptions->pluginName = plugin->name();
+                        options->filterOptions.append( filterOptions );
+                    }
+                }
+            }
+
+            if( saveLastUsed )
+            {
+                config->data.general.lastProfile = currentProfile();
+                saveCustomProfile( true );
+                config->data.general.lastFormat = cFormat->currentText();
+            }
         }
     }
 
@@ -328,17 +384,55 @@ bool OptionsDetailed::setCurrentConversionOptions( ConversionOptions *options )
 
     cFormat->setCurrentIndex( cFormat->findText(options->codecName) );
     formatChanged( cFormat->currentText() );
-    cPlugin->setCurrentIndex( cPlugin->findText(options->pluginName) );
-    encoderChanged( cPlugin->currentText() );
+    if( options->codecName != "wav" )
+    {
+        cPlugin->setCurrentIndex( cPlugin->findText(options->pluginName) );
+        encoderChanged( cPlugin->currentText() );
+    }
     outputDirectory->setMode( (OutputDirectory::Mode)options->outputDirectoryMode );
     outputDirectory->setDirectory( options->outputDirectory );
     cReplayGain->setChecked( options->replaygain );
-//     cBpm->setChecked( options->bpm );
 
-    if( wPlugin )
-        return qobject_cast<CodecWidget*>(wPlugin)->setCurrentConversionOptions( options );
+    bool succeeded = true;
+
+    if( options->codecName == "wav" )
+        succeeded = true;
+    else if( wPlugin )
+        succeeded = wPlugin->setCurrentConversionOptions( options );
     else
-        return false;
+        succeeded = false;
+
+    QStringList usedFilter;
+    foreach( FilterOptions *filterOptions, options->filterOptions )
+    {
+        bool filterSucceeded = false;
+        for( int i=0; i<wFilter.size(); i++ )
+        {
+            FilterWidget *widget = wFilter.keys().at(i);
+            FilterPlugin *plugin = wFilter.values().at(i);
+            if( widget && plugin && filterOptions->pluginName == plugin->name() )
+            {
+                filterSucceeded = widget->setCurrentFilterOptions( filterOptions );
+                usedFilter.append( filterOptions->pluginName );
+                break;
+            }
+        }
+        if( !filterSucceeded )
+            succeeded = false;
+    }
+    // if a filter is disabled, its FilterOptions is 0 thus it won't be added to ConversionOptions, but we need to update the widget so it won't show false data
+    for( int i=0; i<wFilter.size(); i++ )
+    {
+        FilterWidget *widget = wFilter.keys().at(i);
+        FilterPlugin *plugin = wFilter.values().at(i);
+        if( widget && plugin && !usedFilter.contains(plugin->name()) )
+        {
+            widget->setCurrentFilterOptions( 0 );
+            break;
+        }
+    }
+
+    return succeeded;
 }
 
 bool OptionsDetailed::saveCustomProfile( bool lastUsed )
@@ -353,7 +447,7 @@ bool OptionsDetailed::saveCustomProfile( bool lastUsed )
         else
         {
             bool ok;
-            QString profileName = KInputDialog::getText( i18n("New profile"), i18n("Enter a name for the new profile:"), "", &ok );
+            profileName = KInputDialog::getText( i18n("New profile"), i18n("Enter a name for the new profile:"), "", &ok );
             if( !ok )
                 return false;
         }
@@ -384,77 +478,77 @@ bool OptionsDetailed::saveCustomProfile( bool lastUsed )
             return false;
         }
 
-        QDomDocument data = qobject_cast<CodecWidget*>(wPlugin)->customProfile();
-        QDomElement root = data.documentElement();
-        root.setAttribute("pluginName",currentPlugin->name());
-        root.setAttribute("profileName",profileName);
-        QDomElement outputOptions = data.createElement("outputOptions");
-        outputOptions.setAttribute("mode",outputDirectory->mode());
-        outputOptions.setAttribute("directory",outputDirectory->directory());
-        root.appendChild(outputOptions);
-        QDomElement features = data.createElement("features");
-        features.setAttribute("replaygain",cReplayGain->isChecked()&&cReplayGain->isEnabled());
-        root.appendChild(features);
+        QDomDocument list("soundkonverter_profilelist");
+        QDomElement root;
+        bool profileFound = false;
 
-        for( int i=0; i<config->data.profiles.count(); i++ )
+        QFile listFile( KStandardDirs::locateLocal("data","soundkonverter/profiles.xml") );
+        if( listFile.open( QIODevice::ReadOnly ) )
         {
-            if( config->data.profiles.at(i).profileName == profileName )
+            if( list.setContent( &listFile ) )
             {
-                int ret;
-                if( lastUsed )
+                root = list.documentElement();
+                if( root.nodeName() == "soundkonverter" && root.attribute("type") == "profilelist" )
                 {
-                    ret = KMessageBox::Yes;
-                }
-                else
-                {
-                    ret = KMessageBox::questionYesNo( this, i18n("A profile with this name already exists.\n\nDo you want to overwrite the existing one?"), i18n("Profile already exists") );
-                }
-                if( ret == KMessageBox::Yes )
-                {
-                    config->data.profiles[i].pluginName = currentPlugin->name();
-                    config->data.profiles[i].codecName = cFormat->currentText();
-                    config->data.profiles[i].data = data;
-                    QFile profileFile( KStandardDirs::locateLocal("data",QString("soundkonverter/profiles/")) + config->data.profiles.at(i).fileName );
-                    if( profileFile.open( QIODevice::WriteOnly ) )
+                    QDomNodeList conversionOptionsElements = root.elementsByTagName("conversionOptions");
+                    for( int i=0; i<conversionOptionsElements.count(); i++ )
                     {
-                        QTextStream stream(&profileFile);
-                        stream << data;
-                        profileFile.close();
-                        updateProfiles();
-                        emit customProfilesEdited();
-                        return true;
+                        if( conversionOptionsElements.at(i).toElement().attribute("profileName") == profileName )
+                        {
+                            int ret;
+                            if( lastUsed )
+                                ret = KMessageBox::Yes;
+                            else
+                                ret = KMessageBox::questionYesNo( this, i18n("A profile with this name already exists.\n\nDo you want to overwrite the existing one?"), i18n("Profile already exists") );
+
+                            if( ret == KMessageBox::Yes )
+                            {
+                                ConversionOptions *conversionOptions = currentConversionOptions( false );
+                                delete config->data.profiles[profileName];
+                                config->data.profiles[profileName] = conversionOptions;
+                                root.removeChild(conversionOptionsElements.at(i));
+                                QDomElement profileElement = conversionOptions->toXml(list);
+                                profileElement.setAttribute("profileName",profileName);
+                                root.appendChild(profileElement);
+                                profileFound = true;
+                                break;
+                            }
+                            else
+                            {
+                                return false;
+                            }
+                        }
                     }
-                    else
-                    {
-                        return false;
-                    }
-                }
-                else
-                {
-                    return false;
                 }
             }
+            listFile.close();
         }
 
-        Config::ProfileData profile;
-        int i=1;
-        do {
-            profile.fileName = QString::number(i);
-            i++;
-        } while( QFile::exists( KStandardDirs::locateLocal("data",QString("soundkonverter/profiles/")) + profile.fileName) );
-        profile.pluginName = currentPlugin->name();
-        profile.profileName = profileName;
-        profile.codecName = cFormat->currentText();
-        profile.data = data;
-        config->data.profiles += profile;
-        QFile profileFile( KStandardDirs::locateLocal("data",QString("soundkonverter/profiles/")) + profile.fileName );
-        if( profileFile.open( QIODevice::WriteOnly ) )
+        if( listFile.open( QIODevice::WriteOnly ) )
         {
-            QTextStream stream(&profileFile);
-            stream << data;
-            profileFile.close();
+            if( list.childNodes().isEmpty() )
+            {
+                root = list.createElement("soundkonverter");
+                root.setAttribute("type","profilelist");
+                list.appendChild(root);
+            }
+
+            if( !profileFound )
+            {
+                ConversionOptions *conversionOptions = currentConversionOptions( false );
+                config->data.profiles[profileName] = conversionOptions;
+                QDomElement profileElement = conversionOptions->toXml(list);
+                profileElement.setAttribute("profileName",profileName);
+                root.appendChild(profileElement);
+            }
+
             updateProfiles();
             emit customProfilesEdited();
+
+            QTextStream stream(&listFile);
+            stream << list.toString();
+            listFile.close();
+
             return true;
         }
         else
@@ -471,61 +565,31 @@ bool OptionsDetailed::saveCustomProfile( bool lastUsed )
 void OptionsDetailed::loadCustomProfileButtonClicked()
 {
     const QString profile = qobject_cast<QAction*>(QObject::sender())->text().replace("&","");
-    loadCustomProfile( profile );
-}
-
-bool OptionsDetailed::loadCustomProfile( const QString& profile )
-{
-    if( wPlugin && currentPlugin )
-    {
-        for( int i=0; i<config->data.profiles.count(); i++ )
-        {
-            if( config->data.profiles.at(i).profileName == profile && config->data.profiles.at(i).pluginName == currentPlugin->name() )
-            {
-                if( cFormat->currentText() != config->data.profiles.at(i).codecName )
-                {
-                    cFormat->setCurrentIndex( cFormat->findText(config->data.profiles.at(i).codecName) );
-                    formatChanged( cFormat->currentText() );
-                }
-                if( cPlugin->currentText() != config->data.profiles.at(i).pluginName )
-                {
-                    cPlugin->setCurrentIndex( cPlugin->findText(config->data.profiles.at(i).pluginName) );
-                    encoderChanged( cPlugin->currentText() );
-                }
-                QDomElement root = config->data.profiles.at(i).data.documentElement();
-                QDomElement outputOptions = root.elementsByTagName("outputOptions").at(0).toElement();
-                if( profile != "soundkonverter_last_used" )
-                {
-                    outputDirectory->setMode( (OutputDirectory::Mode)outputOptions.attribute("mode").toInt() );
-                    outputDirectory->setDirectory( outputOptions.attribute("directory") );
-                }
-                QDomElement features = root.elementsByTagName("features").at(0).toElement();
-                cReplayGain->setChecked( features.attribute("replaygain").toInt() );
-                return qobject_cast<CodecWidget*>(wPlugin)->setCustomProfile( profile, config->data.profiles.at(i).data );
-            }
-        }
-    }
-
-    return false;
+    setCurrentProfile( profile );
 }
 
 QString OptionsDetailed::currentProfile()
 {
     if( wPlugin )
-        return qobject_cast<CodecWidget*>(wPlugin)->currentProfile();
+        return wPlugin->currentProfile();
     else
         return "";
 }
 
-void OptionsDetailed::setCurrentProfile( const QString& profile )
+bool OptionsDetailed::setCurrentProfile( const QString& profile )
 {
-    if( !wPlugin )
-        return;
-
-    if( !qobject_cast<CodecWidget*>(wPlugin)->setCurrentProfile(profile) )
+    if( config->data.profiles.keys().contains(profile) )
     {
-        // TODO find a plugin that supports the profile (eg. hybrid)
+        ConversionOptions *conversionOptions = config->data.profiles.value( profile );
+        if( conversionOptions )
+            return setCurrentConversionOptions( conversionOptions );
     }
+    else if( wPlugin )
+    {
+        return wPlugin->setCurrentProfile( profile );
+    }
+
+    return false;
 }
 
 QString OptionsDetailed::currentFormat()
